% feature_map - Explicit feature map for polynomial and Gaussian kernels.
%           
%
% DESCRIPTION:
% 
%   Eksplicitne feature mape su dobivene za sljedece kernele:
%
%       polinomski:
%
%           k(x,y) = ( <x,y> + c )^d
%
%       Gaussian:
%
%           k(x,y) = exp( |x-y|^2 / (2*sigma^2) )
%
%      Vandermonde:
%           Fi(x)=x.^d
%
% INPUTS:
%   X       - Stupci su uzorci, mapiranje se radi za svaki stupac posebno.
%   kernel  - Odabir kernela.
%               'polynomial'
%               'gaussian'
%               ' Vandermonde'
%
%   var1    - Parametar kernela
%               za poly kernel:     stupanj (d = var1)
%               za gaussov kernel:  sigma   (sigma = var1)
%   var2    - Parametar kernela
%               za poly kernel:     offset  (c = var2)
%               za gaussov kernel:  red mapiranja (maksimalni stupanj u aproksimaciji)  (p_max = var2)
%
%
% REFERENCES:
%
%
% NOTES:
%    1. Ova funkcija dobro radi za male dimenzije ulaznog vektora, kada se i isplati koristiti
%       explicitna preslikavanja.
%
%
% COPYRIGHT: A.Jukic, I. Kopriva, ZLAIR, IRB, Zagreb, Croatia
% REVISION:  0.0.2
% DATE:      2013/1729; 2012/12/18; 2010/11/25
% 

function Xexp = feature_map(X,kernel,var1,var2)

% INITIALIZATION

if(var1<=0)
    error('Parameter var1 must be positive.');
end

% size of input data
N = size(X,1);
T = size(X,2);

% parameters of kernel
switch(kernel)
    case 'polynomial'
        d = var1;       % degree
        c = var2;       % offset
        if(c<0)
            error('Parameter c (offset) must be nonnegative.');
        end
    
    case 'gaussian'
        sigma = var1;   
        p_max = var2;   % order of approximation
        if(p_max<0)
            error('Parameter p_max must be positive.');
        end
       
    case 'Vandermonde'
        d = var1;       % degree
              
    otherwise
        error('Unknown kernel')
end


% NONLINEAR EXPANSION

if( strcmp(kernel,'polynomial') && c==0 )

    % create all combinations of indices and choose only those with sum equal to d
    k   = combn(0:d,N);
    ind = sum(k,2)==d;
    k   = k(ind,:);
    % sort multi-indices
    [tmp, ind] = sort( max(k,[],2) );
    k = k( ind(end:-1:1), : );
    % create expansion
    Xexp = ones( size(k,1), T );
    for i=1:size(k,1)
        for j=1:N
            Xexp(i,:) = Xexp(i,:) .* X(j,:).^k(i,j);
        end
        coeff   = multinomial(d,k(i,:));
        Xexp(i,:) = sqrt(coeff) * Xexp(i,:);
    end
    
elseif( strcmp(kernel,'polynomial') && c~=0 )

    % create all combinations of indices and choose only those with sum equal to d
    k   = combn(0:d,N+1);
    ind = sum(k,2)==d;
    k   = k(ind,:);
    % sort multi-indices
    [tmp, ind] = sort( max(k(:,1:N),[],2) );
    k = k( ind(end:-1:1), : );
    % create expansion
    Xexp = ones( size(k,1), T );
    for i=1:size(k,1)
        for j=1:N
            Xexp(i,:) = Xexp(i,:) .* X(j,:).^k(i,j);
        end
        coeff     = multinomial(d,k(i,:)) * c^k(i,N+1);
        Xexp(i,:) = sqrt(coeff) * Xexp(i,:);
    end
    
elseif( strcmp(kernel,'gaussian') )

    % Feature map for Gaussian kernel as in (Minh et al., 2006)
    p   = combn(0:p_max,N);
    if N==1
        p=p';
    end
    ind = sum(p,2) <= p_max;
    p   = p(ind,:);
    % sort multi-indices
    [tmp, ind] = sort( max(p,[],2) );
    p = p( ind, : );
    % create expansion
    e_norm2 = exp( -sum(X.^2)/(2*sigma^2) );
    Xexp = ones( size(p,1), T );
    
    for i=1:size(p,1)
        kk = sum(p(i,:));
        alpha = p(i,:);
        % pokupi sve potencije koje su zastupljene u alpha
        fct=1;
        for j=1:N
            Xexp(i,:) = Xexp(i,:) .* X(j,:).^alpha(j);
            fct=fct*factorial(alpha(j));
        end
        %coeff = sqrt( sigma.^(-2*kk) * power(2,kk) / fct); % Ova varijanta
        %    je za k(x,y) = exp( |x-y|^2 / (2*sigma^2) ), samo onda je scaling factor: exp( -sum(X.^2)/(sigma^2) );  
        coeff = sqrt( sigma.^(-2*kk) / fct);
        Xexp(i,:) = coeff * Xexp(i,:);
        Xexp(i,:) = e_norm2 .* Xexp(i,:);
    end   
elseif ( strcmp(kernel,'Vandermonde') )
    for j=1:size(X,1)   
        for i=0:d
             Xexp((j-1)*(d+1)+i+1,:)=power(X(j,:),i);
        end
    end
end



